"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var rootComponent_1 = require("../shared/rootComponent");
var mediaGlobalService_1 = require("../shared/mediaGlobalService");
var overlayService_1 = require("../shared/overlayService");
var mediaWidth_1 = require("../shared/mediaWidth");
var dropdownFactory_1 = require("../shared/dropdownFactory");
var resize_observer_polyfill_1 = require("resize-observer-polyfill");
var debounceService_1 = require("../shared/debounceService");
var outlineComponent_1 = require("./outlineComponent");
var articleLinksComponent_1 = require("./articleLinksComponent");
var articleMenuHeaderComponent_1 = require("./articleMenuHeaderComponent");
var menuMode_1 = require("../shared/menuMode");
var ArticleMenuComponent = /** @class */ (function (_super) {
    __extends(ArticleMenuComponent, _super);
    function ArticleMenuComponent(mediaGlobalService, outlineComponent, articleLinksComponent, articleMenuHeaderComponent, debounceService, overlayService, dropdownFactory) {
        var _this = _super.call(this) || this;
        _this.overlayClickListener = function () {
            if (_this._dropdown.isExpanded()) {
                _this._dropdown.collapseWithAnimation();
                _this._overlayService.deactivateOverlay();
            }
        };
        _this.buttonClickListener = function () {
            _this._dropdown.toggleWithAnimation();
            if (_this._dropdown.isExpanded()) {
                _this._overlayService.activateOverlay(_this._articleMenuElement);
                _this.updateDropdown();
                _this._outlineComponent.updateDropdownKnob();
            }
            else {
                _this._overlayService.deactivateOverlay();
            }
        };
        _this.outlineAnchorClickListener = function () {
            if (_this._mediaGlobalService.mediaWidthIs(mediaWidth_1.MediaWidth.narrow)) {
                _this._overlayService.deactivateOverlay();
                _this._dropdown.collapseWithAnimation();
            }
        };
        _this.onChangedToDropdownListener = function (init) {
            _this._menuMode = menuMode_1.MenuMode.dropdown;
            if (!init) {
                _this._bodyResizeObserver.unobserve(document.body);
                window.removeEventListener('resize', _this.updateSideMenu);
                window.removeEventListener('scroll', _this.updateSideMenu);
                _this.resetSideMenu();
            }
            _this._dropdown.collapseWithoutAnimation();
            _this._overlayService.addClickListener(_this.overlayClickListener);
            window.addEventListener('resize', _this.updateDropdown);
        };
        _this.onChangedToSideMenuListener = function (init) {
            _this._menuMode = menuMode_1.MenuMode.sideMenu;
            if (!init) {
                window.removeEventListener('resize', _this.updateDropdown);
                _this._overlayService.removeClickListener(_this.overlayClickListener);
            }
            _this.resetDropdown();
            window.addEventListener('resize', _this.updateSideMenu);
            window.addEventListener('scroll', _this.updateSideMenuDeferred);
            //Makes initial call to updateToc (initial call is defined in the spec) - https://github.com/WICG/ResizeObserver/issues/8, unfortunately, call is delayed
            _this._bodyResizeObserver.observe(document.body);
            _this.updateSideMenu();
        };
        _this.updateDropdown = function () {
            if (_this._menuMode === menuMode_1.MenuMode.dropdown && _this._dropdown.isExpanded()) { // Edge fires resize listeners even after they have been removed
                // TODO figure out some way to do this using css - hard to use calc since 100vh is fixed on many mobile browsers > https://developers.google.com/web/updates/2016/12/url-bar-resizing
                _this._linksAndTocInnerWrapperElement.style.maxHeight = window.innerHeight - ArticleMenuComponent_1.HEADER_HEIGHT + "px";
            }
        };
        _this.resetDropdown = function () {
            _this._linksAndTocInnerWrapperElement.style.maxHeight = '';
            if (_this._dropdown.isExpanded()) {
                _this._overlayService.deactivateOverlay(false);
            }
            _this._dropdown.reset();
        };
        _this.updateSideMenu = function () {
            if (_this._menuMode === menuMode_1.MenuMode.sideMenu) { // Necessary because bodyResizeObserver fires after unobserve
                var articleMenuTop = _this._articleMenuElement.getBoundingClientRect().top;
                var footerTop = _this._footerElement.getBoundingClientRect().top;
                var outlineHeight = (footerTop > window.innerHeight ? window.innerHeight : footerTop)
                    - ArticleMenuComponent_1.VERTICAL_GAP
                    - _this._linksAndOutlineLevel1Height
                    - Math.max(ArticleMenuComponent_1.VERTICAL_GAP, articleMenuTop);
                // Tried setting bottom, max-height, both don't work on edge - scroll bar doesn't go away even when height is greater than 
                // menu height. This works.
                _this._outlineScrollable.style.height = outlineHeight + "px";
            }
            _this._updateSideMenuQueued = false;
        };
        _this.resetSideMenu = function () {
            _this._outlineScrollable.style.height = '';
        };
        // Fixes smooth-scroll/chrome issue. Scroll event fires before raf callbacks are called, since smooth-scroll uses raf to scroll, footer top is not accurate until after smooth-scroll's raf callback
        // fires
        _this.updateSideMenuDeferred = function () {
            if (!_this._updateSideMenuQueued) {
                requestAnimationFrame(_this.updateSideMenu);
                _this._updateSideMenuQueued = true;
            }
        };
        _this._overlayService = overlayService;
        _this._mediaGlobalService = mediaGlobalService;
        _this._debounceService = debounceService;
        _this._dropdownFactory = dropdownFactory;
        _this._outlineComponent = outlineComponent;
        _this.addChildComponents(outlineComponent, articleLinksComponent, articleMenuHeaderComponent);
        return _this;
    }
    ArticleMenuComponent_1 = ArticleMenuComponent;
    ArticleMenuComponent.prototype.setupImmediate = function () {
        this._articleMenuElement = document.getElementById('article-menu');
        if (this.enabled()) {
            this.childComponentsSetupImmediate();
            this._bodyResizeObserver = new resize_observer_polyfill_1.default(this._debounceService.createTimeoutDebounceFunction(this.updateSideMenu, ArticleMenuComponent_1.DEBOUNCE_TIME));
        }
    };
    ArticleMenuComponent.prototype.enabled = function () {
        return this._articleMenuElement ? true : false;
    };
    ArticleMenuComponent.prototype.setupOnDomContentLoaded = function () {
        this.childComponentsSetupOnDomContentLoaded();
        this._linksAndTocOuterWrapperElement = document.getElementById('article-links-and-outline-outer-wrapper');
        this._linksAndTocInnerWrapperElement = document.getElementById('article-links-and-outline-inner-wrapper');
        this._footerElement = document.getElementById('page-footer');
        this._outlineScrollable = document.getElementById('outline-scrollable');
        this._headerButtonElement = document.getElementById('article-menu-header-button');
        this._dropdown = this._dropdownFactory.build(this._linksAndTocOuterWrapperElement, this._linksAndTocInnerWrapperElement, this._headerButtonElement, this._articleMenuElement);
    };
    ArticleMenuComponent.prototype.setupOnLoad = function () {
        var outlineTitleElement = document.querySelector('#outline > .level-1');
        var outlineTitleComputedStyle = getComputedStyle(outlineTitleElement);
        var articleLinksElement = document.getElementById('article-links');
        var articleLinksComputedStyle = getComputedStyle(articleLinksElement);
        var outlineComputedStyle = getComputedStyle(this._outlineScrollable);
        this._linksAndOutlineLevel1Height = parseFloat(articleLinksComputedStyle.marginBottom) +
            parseFloat(articleLinksComputedStyle.height) +
            parseFloat(outlineTitleComputedStyle.height) +
            parseFloat(outlineComputedStyle.marginTop) +
            articleLinksComponent_1.default.TOP_NEGATIVE_MARGIN;
        this._headerButtonElement.addEventListener('click', this.buttonClickListener);
        var outlineAnchorElements = this._outlineComponent.getAnchorElements();
        if (outlineAnchorElements) {
            for (var i = 0; i < outlineAnchorElements.length; i++) {
                outlineAnchorElements[i].addEventListener('click', this.outlineAnchorClickListener);
            }
            outlineTitleElement.addEventListener('click', this.outlineAnchorClickListener);
        }
        this._mediaGlobalService.addChangedToListener(this.onChangedToDropdownListener, mediaWidth_1.MediaWidth.narrow);
        this._mediaGlobalService.addChangedFromListener(this.onChangedToSideMenuListener, mediaWidth_1.MediaWidth.narrow);
        // outlineComponent's update knob methods must run after updateToc (incase updating outline height causes wrapping, in turn causing anchor bounding rects to change)
        this.childComponentsSetupOnLoad();
    };
    var ArticleMenuComponent_1;
    ArticleMenuComponent.VERTICAL_GAP = 23;
    ArticleMenuComponent.HEADER_HEIGHT = 37;
    ArticleMenuComponent.DEBOUNCE_TIME = 150;
    ArticleMenuComponent = ArticleMenuComponent_1 = __decorate([
        inversify_1.injectable(),
        __param(0, inversify_1.inject('GlobalService')), __param(0, inversify_1.named('MediaGlobalService')),
        __metadata("design:paramtypes", [mediaGlobalService_1.default,
            outlineComponent_1.default,
            articleLinksComponent_1.default,
            articleMenuHeaderComponent_1.default,
            debounceService_1.default,
            overlayService_1.default,
            dropdownFactory_1.default])
    ], ArticleMenuComponent);
    return ArticleMenuComponent;
}(rootComponent_1.default));
exports.default = ArticleMenuComponent;
//# sourceMappingURL=articleMenuComponent.js.map