"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var rootComponent_1 = require("../shared/rootComponent");
var mediaGlobalService_1 = require("../shared/mediaGlobalService");
var overlayService_1 = require("../shared/overlayService");
var mediaWidth_1 = require("../shared/mediaWidth");
var categoryPagesComponent_1 = require("./categoryPagesComponent");
var categoryPagesFilterComponent_1 = require("./categoryPagesFilterComponent");
var dropdownFactory_1 = require("../shared/dropdownFactory");
var resize_observer_polyfill_1 = require("resize-observer-polyfill");
var debounceService_1 = require("../shared/debounceService");
var menuMode_1 = require("../shared/menuMode");
var CategoryMenuComponent = /** @class */ (function (_super) {
    __extends(CategoryMenuComponent, _super);
    function CategoryMenuComponent(mediaGlobalService, debounceService, categoryPagesComponent, categoryPagesFilterComponent, overlayService, dropdownFactory) {
        var _this = _super.call(this) || this;
        _this.overlayClickListener = function () {
            if (_this._dropdown.isExpanded()) {
                _this._dropdown.collapseWithAnimation();
                _this._overlayService.deactivateOverlay();
            }
        };
        _this.buttonClickListener = function () {
            _this._dropdown.toggleWithAnimation();
            if (_this._dropdown.isExpanded()) {
                _this._overlayService.activateOverlay(_this._categoryMenuElement);
                _this.updateDropdown();
            }
            else {
                _this._overlayService.deactivateOverlay();
            }
        };
        _this.onChangedToDropdownListener = function (init) {
            _this._menuMode = menuMode_1.MenuMode.dropdown;
            if (!init) {
                _this._bodyResizeObserver.unobserve(document.body);
                window.removeEventListener('resize', _this.updateSideMenu);
                window.removeEventListener('scroll', _this.updateSideMenu);
                _this.resetSideMenu();
            }
            _this._dropdown.collapseWithoutAnimation();
            window.addEventListener('resize', _this.updateDropdown);
            _this._overlayService.addClickListener(_this.overlayClickListener);
            _this.updateDropdown();
        };
        _this.onChangedToSideMenuListener = function (init) {
            _this._menuMode = menuMode_1.MenuMode.sideMenu;
            if (!init) {
                window.removeEventListener('resize', _this.updateDropdown);
                _this._overlayService.removeClickListener(_this.overlayClickListener);
            }
            _this.resetDropdown();
            window.addEventListener('resize', _this.updateSideMenu);
            window.addEventListener('scroll', _this.updateSideMenuDeferred);
            //Makes initial call to updateSideMenu (initial call is defined in the spec) - https://github.com/WICG/ResizeObserver/issues/8, unfortunately it is delayed
            _this._bodyResizeObserver.observe(document.body);
            _this.updateSideMenu();
        };
        // | MediaWidth | inCore2 | collapsed |
        // | narrow     | y/n     | y         |
        // | medium     | y       | y         |
        // | medium     | n       | n         |
        // | wide       | y/n     | n         |
        _this.updateDropdown = function () {
            if (_this._menuMode === menuMode_1.MenuMode.dropdown && _this._dropdown.isExpanded()) { // Edge fires resize listeners even after they have been removed
                // Update wrapper maxheight so it is scrollable as a dropdown menu
                _this._pagesInnerWrapperElement.style.maxHeight = window.innerHeight - CategoryMenuComponent_1.HEADER_HEIGHT + "px";
            }
        };
        _this.resetDropdown = function () {
            _this._pagesInnerWrapperElement.style.maxHeight = '';
            if (_this._dropdown.isExpanded()) {
                _this._overlayService.deactivateOverlay(false);
            }
            _this._dropdown.reset();
        };
        // This function cannot be part of pages component because pages filter and category menu element state is required
        _this.updateSideMenu = function () {
            if (_this._menuMode === menuMode_1.MenuMode.sideMenu) { // ResizeObserver.Unobserve fires listeners in chrome
                var categoryMenuTop = _this._categoryMenuElement.getBoundingClientRect().top;
                var footerTop = _this._footerElement.getBoundingClientRect().top;
                var pagesHeight = (footerTop > window.innerHeight ? window.innerHeight : footerTop)
                    - CategoryMenuComponent_1.VERTICAL_GAP
                    - _this._filterHeight
                    - Math.max(CategoryMenuComponent_1.VERTICAL_GAP, categoryMenuTop);
                // Tried setting bottom, max-height, both don't work on edge - scroll bar doesn't go away even when height is greater than 
                // menu height. This works.
                _this._categoryPagesElement.style.height = pagesHeight + "px";
            }
            _this._updateSideMenuQueued = false;
        };
        _this.resetSideMenu = function () {
            _this._categoryPagesElement.style.height = '';
        };
        _this.updateSideMenuDeferred = function () {
            if (!_this._updateSideMenuQueued) {
                requestAnimationFrame(_this.updateSideMenu);
                _this._updateSideMenuQueued = true;
            }
        };
        _this._overlayService = overlayService;
        _this._mediaGlobalService = mediaGlobalService;
        _this._debounceService = debounceService;
        _this._dropdownFactory = dropdownFactory;
        _this.addChildComponents(categoryPagesFilterComponent, categoryPagesComponent);
        return _this;
    }
    CategoryMenuComponent_1 = CategoryMenuComponent;
    CategoryMenuComponent.prototype.enabled = function () {
        return this._categoryMenuElement ? true : false;
    };
    CategoryMenuComponent.prototype.setupImmediate = function () {
        this._categoryMenuElement = document.getElementById('category-menu');
        if (this.enabled()) {
            this.childComponentsSetupImmediate();
            this._bodyResizeObserver = new resize_observer_polyfill_1.default(this._debounceService.createTimeoutDebounceFunction(this.updateSideMenu, CategoryMenuComponent_1.DEBOUNCE_TIME));
        }
    };
    CategoryMenuComponent.prototype.setupOnDomContentLoaded = function () {
        this.childComponentsSetupOnDomContentLoaded();
        this._pagesInnerWrapperElement = document.getElementById('category-pages-inner-wrapper');
        this._categoryPagesElement = document.getElementById('category-pages');
        this._footerElement = document.getElementById('page-footer');
        this._headerButtonElement = document.getElementById('category-menu-header-button');
        this._pagesOuterWrapperElement = document.getElementById('category-pages-outer-wrapper');
        this._dropdown = this._dropdownFactory.build(this._pagesOuterWrapperElement, this._pagesInnerWrapperElement, this._headerButtonElement, this._categoryMenuElement);
    };
    CategoryMenuComponent.prototype.setupOnLoad = function () {
        var filterElement = document.getElementById('category-pages-filter');
        var filterComputedStyle = getComputedStyle(filterElement);
        // Does not change
        this._filterHeight = parseFloat(filterComputedStyle.marginBottom) + parseFloat(filterComputedStyle.height);
        this.childComponentsSetupOnLoad();
        this._headerButtonElement.addEventListener('click', this.buttonClickListener);
        var inCoreOuter = this._categoryMenuElement.parentElement.getAttribute('id') === 'core-outer';
        if (inCoreOuter) {
            this._mediaGlobalService.addChangedFromListener(this.onChangedToDropdownListener, mediaWidth_1.MediaWidth.wide);
            this._mediaGlobalService.addChangedToListener(this.onChangedToSideMenuListener, mediaWidth_1.MediaWidth.wide);
        }
        else {
            this._mediaGlobalService.addChangedFromListener(this.onChangedToSideMenuListener, mediaWidth_1.MediaWidth.narrow);
            this._mediaGlobalService.addChangedToListener(this.onChangedToDropdownListener, mediaWidth_1.MediaWidth.narrow);
        }
    };
    var CategoryMenuComponent_1;
    CategoryMenuComponent.VERTICAL_GAP = 23;
    CategoryMenuComponent.HEADER_HEIGHT = 37;
    CategoryMenuComponent.DEBOUNCE_TIME = 150;
    CategoryMenuComponent = CategoryMenuComponent_1 = __decorate([
        inversify_1.injectable(),
        __param(0, inversify_1.inject('GlobalService')), __param(0, inversify_1.named('MediaGlobalService')),
        __metadata("design:paramtypes", [mediaGlobalService_1.default,
            debounceService_1.default,
            categoryPagesComponent_1.default,
            categoryPagesFilterComponent_1.default,
            overlayService_1.default,
            dropdownFactory_1.default])
    ], CategoryMenuComponent);
    return CategoryMenuComponent;
}(rootComponent_1.default));
exports.default = CategoryMenuComponent;
//# sourceMappingURL=categoryMenuComponent.js.map