"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var collapsibleMenuFactory_1 = require("../shared/collapsibleMenuFactory");
var heightService_1 = require("../shared/heightService");
var CategoryPagesComponent = /** @class */ (function () {
    function CategoryPagesComponent(collapsibleMenuFactory, heightService) {
        this._collapsibleMenuFactory = collapsibleMenuFactory;
        this._heightService = heightService;
    }
    CategoryPagesComponent.prototype.setupImmediate = function () {
    };
    CategoryPagesComponent.prototype.setupOnDomContentLoaded = function () {
        this._categoryPagesElement = document.getElementById('category-pages');
        this.collapsibleMenu = this._collapsibleMenuFactory.build(this._categoryPagesElement);
    };
    CategoryPagesComponent.prototype.setupOnLoad = function () {
        this.expandInitiallyExpandedLIElements();
    };
    CategoryPagesComponent.prototype.expandInitiallyExpandedLIElements = function () {
        var activePageElement = this._categoryPagesElement.querySelector('.active');
        // Page may not be listed in category menu
        if (!activePageElement) {
            return;
        }
        var currentElement = activePageElement.parentElement;
        var initiallyExpandedLIElements = [];
        while (currentElement != this._categoryPagesElement) {
            if (currentElement.tagName === 'LI' && currentElement.classList.contains('expandable')) {
                initiallyExpandedLIElements.push(currentElement);
            }
            currentElement = currentElement.parentElement;
        }
        // There may be no LI elements to expand
        if (initiallyExpandedLIElements.length == 0) {
            return;
        }
        // If an element is nested in another element and a height transition is started for both at the same
        // time, the outer element only transitions to its height. This is because 
        // toggleHeightForTransition has no way to know the final heights of an element's children. Nested children at
        // the bottom of the outer element are only revealed when its height is set to auto in its transitionend callback.
        // Therefore it is necessary to immediately expand nested elements.
        for (var i = 0; i < initiallyExpandedLIElements.length; i++) {
            var listElement = initiallyExpandedLIElements[i];
            if (i === initiallyExpandedLIElements.length - 1) {
                this._heightService.toggleHeightWithTransition(listElement.querySelector('ul'), listElement);
            }
            else {
                this._heightService.expandHeightWithoutTransition(listElement.querySelector('ul'), listElement);
            }
            // TODO generalize and move to edgeWorkaroundsService
            // Yet another Edge workaround - 
            // On page load, Edge does not rotate the svg until mouse hovers over the li element it is contained in.
            // This is a really dirty temporary fix that forces the rotation.
            var svgElement = listElement.firstElementChild.firstElementChild;
            svgElement.style.transform = 'rotate(90deg)';
            svgElement.style.transform = '';
        }
    };
    CategoryPagesComponent = __decorate([
        inversify_1.injectable(),
        __metadata("design:paramtypes", [collapsibleMenuFactory_1.default,
            heightService_1.default])
    ], CategoryPagesComponent);
    return CategoryPagesComponent;
}());
exports.default = CategoryPagesComponent;
//# sourceMappingURL=categoryPagesComponent.js.map