"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var mediaGlobalService_1 = require("./mediaGlobalService");
var debounceService_1 = require("./debounceService");
var SmoothScroll = require("smooth-scroll");
var mediaWidth_1 = require("./mediaWidth");
var ArticleGlobalService = /** @class */ (function () {
    function ArticleGlobalService(mediaGlobalService, debounceService) {
        var _this = this;
        this.onChangedToNarrowListener = function () {
            if (_this._mediumWideIntersectionObserver) {
                _this._mediumWideIntersectionObserver.disconnect();
            }
            // Set header height
            if (_this._headerEnabled) {
                _this._headerHeight = 37;
            }
            else {
                _this._headerHeight = 0;
            }
            // Create and register intersection observer
            if (!_this._narrowIntersectionObserver) {
                _this._narrowIntersectionObserver = _this.createNarrowIntersectionObserver();
                _this._narrowInterSectionListenerCorrected = false;
            }
            _this.observeSections(_this._narrowIntersectionObserver);
        };
        this.createNarrowIntersectionObserver = function (multiplyByDevicePixelRatio) {
            if (multiplyByDevicePixelRatio === void 0) { multiplyByDevicePixelRatio = false; }
            var topMargin = -_this._headerHeight;
            if (multiplyByDevicePixelRatio) {
                topMargin *= window.devicePixelRatio;
            }
            return new IntersectionObserver(_this.onIntersectionListener, { threshold: 0, rootMargin: topMargin + "px 0px 0px 0px" });
        };
        this.onChangedFromNarrowListener = function () {
            if (_this._narrowIntersectionObserver) {
                _this._narrowIntersectionObserver.disconnect();
            }
            // Set header height
            _this._headerHeight = 0;
            // Create and register intersection observer
            if (!_this._mediumWideIntersectionObserver) {
                _this._mediumWideIntersectionObserver = new IntersectionObserver(_this.onIntersectionListener, { threshold: 0 });
            }
            _this.observeSections(_this._mediumWideIntersectionObserver);
        };
        this.observeSections = function (intersectionObserver) {
            for (var i = 0; i < _this._observedSectionDatas.length; i++) {
                var observedSectionData = _this._observedSectionDatas[i];
                if (observedSectionData.prevNonSectionElement) {
                    intersectionObserver.observe(observedSectionData.prevNonSectionElement);
                }
                intersectionObserver.observe(observedSectionData.element);
            }
        };
        this.onIntersectionListener = function (entries, _) {
            // On Chrome for Android, rootMargins are read as device pixels (as opposed to CSS pixels > window.devicePixelRatio = devicePixels/cssPixels). 
            // To verify whether or not we are dealing with Chrome for Android, we check whether the computed rootBounds.top is equal to what we requested, _headerHeight.
            // If it isn't, we ditch the initial _narrowIntersectionObserver and create a new one with rootMargins multiplied by window.devicePixelRatio.
            if (_this._headerHeight != 0 && // We don't use rootMargins when headerHeight is 0
                !_this._narrowInterSectionListenerCorrected && // Only correct once. Chrome for Anrdoid computes rootBounds.top as ourSpecifiedRootMarginTop/window.devicePixelRatio. Even
                // if we specify outSpecifiedRootMarginTop as _headerHeight * window.devicePixelRatio, rootBounds.top may not === _headerHeight
                // due to the accuracy limitations of floating point calculations.
                entries[0].rootBounds.top !== _this._headerHeight) {
                _this._narrowInterSectionListenerCorrected = true;
                _this._narrowIntersectionObserver.disconnect();
                _this._narrowIntersectionObserver = _this.createNarrowIntersectionObserver(true);
                _this.observeSections(_this._narrowIntersectionObserver);
                return;
            }
            var _loop_1 = function (i) {
                var entry = entries[i];
                _this._observedSectionDatas.some(function (observedSectionData) {
                    // Since threshold is 0, if callback has fired for element, isIntersecting == isVisible.
                    if (observedSectionData.element == entry.target) {
                        observedSectionData.isVisible = entry.isIntersecting;
                        return true;
                    }
                    else if (observedSectionData.prevNonSectionElement == entry.target) {
                        observedSectionData.prevIsVisible = entry.isIntersecting;
                        return true;
                    }
                    return false;
                });
            };
            // Update observed elements isVisible
            for (var i = 0; i < entries.length; i++) {
                _loop_1(i);
            }
            // Don't update active section index if it is fixed
            if (_this._activeSectionIndexFixed) {
                return;
            }
            // Get index of first visible section index
            var activeSectionIndex = _this._observedSectionDatas.findIndex(function (observedSectionData, index) {
                if (observedSectionData.isVisible) {
                    var nextObservedSectionData = void 0;
                    if (observedSectionData.level == 3 || // Highest level
                        index == _this._observedSectionDatas.length - 1 || // Section is the last observed section
                        (nextObservedSectionData = _this._observedSectionDatas[index + 1]).level <= observedSectionData.level || // Next observed section has lower or equal level
                        nextObservedSectionData.prevIsVisible) { // Next observed section isn't the active section yet
                        return true;
                    }
                }
                return false;
            });
            if (_this.setActiveSectionIndex(activeSectionIndex)) {
                _this._updateHistoryDebounced();
            }
        };
        // Chrome and edge have rendering issues if activeHeaderIndex changes rapidly and knob translate/scale are updated rapidly.
        // Fixing activeHeaderIndex on smooth-scroll avoids these rendering issues.
        this.smoothScrollBefore = function () {
            var hash = location.hash;
            _this._activeSectionIndexFixed = true;
            // # or #top navigate to the top of the page - https://developer.mozilla.org/en-US/docs/Web/HTML/Element/a#attr-href.
            // Smooth-scroll used to add a #smooth-scroll-top id to body to scroll to the top, it no longer does that, instead
            // it just scrolls to # or #top - https://github.com/cferdinandi/smooth-scroll/commit/37f579b05f6173bba300777867f6b8b613339662#diff-2fa6e0fecc1866964986277037867a1cR468.
            if (!hash || hash === '#top' || hash === '#') {
                _this.setActiveSectionIndex(0);
                return;
            }
            var bodyBottom = document.body.getBoundingClientRect().bottom;
            for (var i = 0; i < _this._sectionHashes.length; i++) {
                if (hash === _this._sectionHashes[i]) {
                    // Ensure that section can be scrolled to
                    // Note that BoundingClientRect.top does not include top margin.
                    while (bodyBottom - _this._observedSectionDatas[i].element.getBoundingClientRect().top + _this._sectionMarginTops[i] < window.innerHeight - _this._headerHeight) {
                        if (i === 0) { // If we get to 0, it means that even the first section can't be scrolled to
                            break;
                        }
                        i--;
                    }
                    _this.setActiveSectionIndex(i);
                    return;
                }
            }
            // Not navigating to top or an article section (could be a link in the article to another element in the article)
            _this._activeSectionIndexFixed = false;
        };
        this.smoothScrollAfter = function () {
            // Note: scroll listener fires one last time after smoothScrollAfter is called
            _this._activeSectionIndexFixed = false;
        };
        this.updateHistory = function () {
            var url = _this._activeSectionIndex == 0 ? location.pathname : _this._sectionHashes[_this._activeSectionIndex];
            if (url === location.hash) {
                return;
            }
            history.replaceState(null, null, url);
        };
        this._mediaGlobalService = mediaGlobalService;
        this._debounceService = debounceService;
        this._activeSectionIndex = 0; // Should never be undefined
    }
    ArticleGlobalService.prototype.setupOnDomContentLoaded = function () {
        // Do nothing
    };
    ArticleGlobalService.prototype.setupImmediate = function () {
        var _this = this;
        // Setup smooth scroll
        this.setupSmoothScroll();
        // Check if header is enabled
        this._headerEnabled = !!document.getElementById('article-menu-header');
        // Setup section datas for observation
        this._sectionElements = Array.from(document.querySelectorAll('.main-article, .main-article > .flexi-section-block-2, .main-article > .flexi-section-block-2 > .flexi-section-block-3')); // Ignore sections in subtrees
        if (this._sectionElements.length == 1) { // Only article
            this._noOutline = true;
            return;
        }
        else {
            this._noOutline = false;
        }
        this._observedSectionDatas = [];
        this._sectionHashes = [];
        this._sectionElements.forEach(function (sectionElement, index) {
            var level = 3;
            if (sectionElement.classList.contains('main-article')) {
                level = 1;
            }
            else if (sectionElement.classList.contains('flexi-section-block-2')) {
                level = 2;
            }
            _this._observedSectionDatas.push({
                isVisible: false,
                element: sectionElement,
                level: level,
                prevNonSectionElement: sectionElement.previousElementSibling && sectionElement.previousElementSibling.tagName != 'SECTION' ? sectionElement.previousElementSibling : null,
                prevIsVisible: false
            });
            // Set section hashes
            _this._sectionHashes[index] = "#" + sectionElement.id;
        });
        // Misc setup
        this._sectionMarginTops = [];
        this._activeSectionChangedListeners = [];
        this._updateHistoryDebounced = this._debounceService.createTimeoutDebounceFunction(this.updateHistory, 100);
    };
    ArticleGlobalService.prototype.setupOnLoad = function () {
        if (this._noOutline) {
            return;
        }
        this.setupSectionMarginTops();
        this._mediaGlobalService.addChangedToListener(this.onChangedToNarrowListener, mediaWidth_1.MediaWidth.narrow);
        this._mediaGlobalService.addChangedFromListener(this.onChangedFromNarrowListener, mediaWidth_1.MediaWidth.narrow);
        // Custom smooth-scroll events
        document.addEventListener('scrollStart', this.smoothScrollBefore, false);
        document.addEventListener('scrollStop', this.smoothScrollAfter, false);
        // Set initial _activeSectionIndex
        var hash = location.hash;
        if (hash && hash !== '#top' && hash !== '#') {
            var hashContent_1 = hash.substring(1);
            var section = this._sectionElements.find(function (element) { return element.id === hashContent_1; });
            if (section) {
                this._smoothScroll.animateScroll(section); // When article menu header is visible (narrow), if user navigated to #<some element>, the initial page load does not account for header height.
                return;
            }
        }
        this.setActiveSectionIndex(0); // Call even though _activeSectionIndex is 0 by default so that listeners get called
    };
    ArticleGlobalService.prototype.addActiveSectionChangedListener = function (listener, init) {
        if (init === void 0) { init = false; }
        if (this._noOutline) {
            return;
        }
        this._activeSectionChangedListeners.push(listener);
        if (init) {
            listener(this._activeSectionIndex);
        }
    };
    ArticleGlobalService.prototype.removeActiveSectionChangedListener = function (listener) {
        if (this._noOutline) {
            return;
        }
        var index = this._activeSectionChangedListeners.indexOf(listener);
        if (index > -1) {
            this._activeSectionChangedListeners.splice(index, 1);
        }
    };
    ArticleGlobalService.prototype.setupSectionMarginTops = function () {
        for (var i = 0; i < this._observedSectionDatas.length; i++) {
            this._sectionMarginTops.push(parseFloat(getComputedStyle(this._observedSectionDatas[i].element).marginTop));
        }
    };
    ArticleGlobalService.prototype.getActiveHeaderIndex = function () {
        return this._activeSectionIndex;
    };
    ArticleGlobalService.prototype.getSectionElements = function () {
        return this._sectionElements;
    };
    ArticleGlobalService.prototype.setupSmoothScroll = function () {
        this._smoothScroll = new SmoothScroll('a[href*="#"]', {
            speed: 300,
            header: '#article-menu-header' // Smooth-scroll updates header height when page is resized (this was removed in later versions of smooth-scroll, be careful when updating)
        });
    };
    ArticleGlobalService.prototype.setActiveSectionIndex = function (newIndex) {
        if (newIndex === this._activeSectionIndex) {
            return false;
        }
        this._activeSectionIndex = newIndex;
        for (var i = 0; i < this._activeSectionChangedListeners.length; i++) {
            this._activeSectionChangedListeners[i](newIndex);
        }
        return true;
    };
    ArticleGlobalService = __decorate([
        inversify_1.injectable(),
        __param(0, inversify_1.inject('GlobalService')), __param(0, inversify_1.named('MediaGlobalService')),
        __metadata("design:paramtypes", [mediaGlobalService_1.default,
            debounceService_1.default])
    ], ArticleGlobalService);
    return ArticleGlobalService;
}());
exports.default = ArticleGlobalService;
//# sourceMappingURL=articleGlobalService.js.map