"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var collapsibleMenu_1 = require("./collapsibleMenu");
var stringService_1 = require("./stringService");
var heightService_1 = require("./heightService");
var CollapsibleMenuFactory = /** @class */ (function () {
    function CollapsibleMenuFactory(stringService, heightService) {
        this._stringService = stringService;
        this._heightService = heightService;
    }
    CollapsibleMenuFactory.prototype.build = function (rootElement) {
        var rootElementID = rootElement.getAttribute('id');
        var rootLIElements = document.querySelectorAll("#" + rootElementID + " > ul > li");
        var expandableTextElements = document.querySelectorAll("#" + rootElementID + " li.expandable > span, #" + rootElementID + " li.expandable > a");
        var leafTextElements = document.querySelectorAll("#" + rootElementID + " li > span, #" + rootElementID + " li > a");
        this.setTextElementsPadding(leafTextElements, 21); // Leaf text elements have no icon, so we need extra padding to align them with expandable text elements
        this.setTextElementsPadding(expandableTextElements);
        this.registerTopicListeners(expandableTextElements);
        return new collapsibleMenu_1.default(rootElement, rootLIElements, this._stringService, this._heightService);
    };
    CollapsibleMenuFactory.prototype.registerTopicListeners = function (topicElements) {
        var _this = this;
        var _loop_1 = function (i) {
            var topicElement = topicElements[i];
            var clickableElement = void 0;
            // If topic element is span, a click anywhere within the element should expand it
            if (topicElement.tagName.toLowerCase() == "a") {
                clickableElement = topicElement.querySelector('button');
            }
            else {
                clickableElement = topicElement;
            }
            clickableElement.addEventListener('click', function (event) {
                var parentLI = topicElement.parentElement;
                var childUl = parentLI.querySelector('ul');
                _this._heightService.toggleHeightWithTransition(childUl, parentLI);
                event.preventDefault();
                // If event propogates, every parent li.expandable's click listener will
                // be called
                event.stopPropagation();
            });
        };
        for (var i = 0; i < topicElements.length; i++) {
            _loop_1(i);
        }
    };
    CollapsibleMenuFactory.prototype.setTextElementsPadding = function (textElements, extraPaddingLeft) {
        if (extraPaddingLeft === void 0) { extraPaddingLeft = 0; }
        for (var i = 0; i < textElements.length; i++) {
            var element = textElements[i];
            var level = parseInt(element.getAttribute('data-level'));
            element.style.paddingLeft = extraPaddingLeft + (level - 1) * 14 + "px";
        }
    };
    CollapsibleMenuFactory = __decorate([
        inversify_1.injectable(),
        __metadata("design:paramtypes", [stringService_1.default,
            heightService_1.default])
    ], CollapsibleMenuFactory);
    return CollapsibleMenuFactory;
}());
exports.default = CollapsibleMenuFactory;
//# sourceMappingURL=collapsibleMenuFactory.js.map