"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var components = require("../components");
var services = require("../services");
var factories = require("../factories");
var Host = /** @class */ (function () {
    function Host() {
        var _this = this;
        // Called when document has been parsed but resources may not have been loaded
        this.onDomContentLoaded = function () {
            for (var i = 0; i < _this._globalServices.length; i++) {
                _this._globalServices[i].setupOnDomContentLoaded();
            }
            for (var i = 0; i < _this._rootComponents.length; i++) {
                var rootComponent = _this._rootComponents[i];
                if (rootComponent.enabled()) {
                    rootComponent.setupOnDomContentLoaded();
                }
            }
        };
        // Called after document has been parsed and all resources have been loaded.
        // Logic that depends on dimensions of elements must be run here.
        this.onLoad = function () {
            for (var i = 0; i < _this._globalServices.length; i++) {
                _this._globalServices[i].setupOnLoad();
            }
            for (var i = 0; i < _this._rootComponents.length; i++) {
                var rootComponent = _this._rootComponents[i];
                if (rootComponent.enabled()) {
                    rootComponent.setupOnLoad();
                }
            }
        };
        this._container = new inversify_1.Container();
        this.addDefault(this._container);
    }
    Host.prototype.getContainer = function () {
        return this._container;
    };
    Host.prototype.run = function () {
        this._globalServices = this._container.getAll('GlobalService');
        this._rootComponents = this._container.getAll('RootComponent');
        for (var i = 0; i < this._globalServices.length; i++) {
            this._globalServices[i].setupImmediate();
        }
        for (var i = 0; i < this._rootComponents.length; i++) {
            this._rootComponents[i].setupImmediate();
        }
        if (document.readyState === 'interactive' || document.readyState === 'complete') {
            this.onDomContentLoaded();
        }
        else {
            document.addEventListener('DomContentLoaded', this.onDomContentLoaded);
        }
        if (document.readyState === 'complete') {
            this.onLoad();
        }
        else {
            window.addEventListener('load', this.onLoad);
        }
    };
    Host.prototype.addDefault = function (container) {
        // Page header
        container.bind('RootComponent').to(components.PageHeaderComponent).inSingletonScope();
        container.bind(components.SearchComponent).toSelf().inSingletonScope();
        container.bind(components.SearchResultsComponent).toSelf().inSingletonScope();
        // Category menu
        container.bind('RootComponent').to(components.CategoryMenuComponent).inSingletonScope();
        container.bind(components.CategoryPagesComponent).toSelf().inSingletonScope();
        container.bind(components.CategoryPagesFilterComponent).toSelf().inSingletonScope();
        // Article
        container.bind('RootComponent').to(components.ArticleComponent).inSingletonScope();
        // Comments
        container.bind('RootComponent').to(components.CommentsComponent).inSingletonScope();
        // Sorted article list
        container.bind('RootComponent').to(components.SortedArticleListComponent).inSingletonScope();
        // Article menu
        container.bind('RootComponent').to(components.ArticleMenuComponent).inSingletonScope();
        container.bind(components.OutlineComponent).toSelf().inSingletonScope();
        container.bind(components.ArticleLinksComponent).toSelf().inSingletonScope();
        container.bind(components.ArticleMenuHeaderComponent).toSelf().inSingletonScope();
        // Footer
        container.bind('RootComponent').to(components.PageFooterComponent).inSingletonScope();
        // Shared
        container.bind('GlobalService').to(services.ArticleGlobalService).inSingletonScope().whenTargetNamed('ArticleGlobalService');
        container.bind('GlobalService').to(services.MediaGlobalService).inSingletonScope().whenTargetNamed('MediaGlobalService');
        container.bind('GlobalService').to(services.FixesGlobalService).inSingletonScope().whenTargetNamed('FixesGlobalService');
        container.bind(services.CodeService).toSelf().inSingletonScope();
        container.bind(services.DebounceService).toSelf().inSingletonScope();
        container.bind(services.HtmlEncodeService).toSelf().inSingletonScope();
        container.bind(services.OverlayService).toSelf().inSingletonScope();
        container.bind(services.PaginationService).toSelf().inSingletonScope();
        container.bind(services.SearchService).toSelf().inSingletonScope();
        container.bind(services.TooltipService).toSelf().inSingletonScope();
        container.bind(services.HeightService).toSelf().inSingletonScope();
        container.bind(services.StringService).toSelf().inSingletonScope();
        container.bind(services.EasingService).toSelf().inSingletonScope();
        container.bind(factories.TextInputFactory).toSelf().inSingletonScope();
        container.bind(factories.CollapsibleMenuFactory).toSelf().inSingletonScope();
        container.bind(factories.DropdownFactory).toSelf().inSingletonScope();
    };
    return Host;
}());
exports.default = Host;
//# sourceMappingURL=host.js.map