"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var OverlayService = /** @class */ (function () {
    function OverlayService() {
        var _this = this;
        this.onFadedOutListener = function (event) {
            if (event.target === event.currentTarget) {
                event.target.removeEventListener('transitionend', _this.onFadedOutListener, true);
                _this.resetElements();
            }
            event.stopPropagation();
        };
    }
    OverlayService.prototype.activateOverlay = function (focusedElement, disableBodyScroll, fadeIn) {
        if (disableBodyScroll === void 0) { disableBodyScroll = true; }
        if (fadeIn === void 0) { fadeIn = true; }
        if (!this._overlayActive) {
            if (!this._overlayElement) {
                this._overlayElement = document.getElementById('overlay');
            }
            else {
                this._overlayElement.removeEventListener('transitionend', this.onFadedOutListener, true);
            }
            this._overlayActive = true;
            // Update overlay element
            this._overlayElement.style.visibility = 'visible';
            this._overlayElement.style.opacity = '0.7';
            if (!fadeIn) {
                this._overlayElement.style.transition = 'initial';
            }
            else {
                this._overlayElement.style.transition = '';
            }
            // Update focused element
            this._currentFocusedElement = focusedElement;
            this._currentFocusedElement.style.zIndex = '3';
            // Update body
            if (disableBodyScroll) {
                document.body.style.overflow = 'hidden';
            }
        }
        else {
            console.warn("Overlay already active, unable to focus on " + focusedElement + ".");
        }
    };
    OverlayService.prototype.deactivateOverlay = function (fadeOut) {
        if (fadeOut === void 0) { fadeOut = true; }
        if (this._overlayActive) {
            this._overlayElement.removeEventListener('transitionend', this.onFadedOutListener, true);
            this._overlayActive = false;
            this._overlayElement.style.opacity = '0';
            if (!fadeOut) {
                this.resetElements();
            }
            else {
                this._overlayElement.addEventListener('transitionend', this.onFadedOutListener, true);
            }
        }
        else {
            console.warn("Overlay inactive, nothing to deactivate.");
        }
    };
    OverlayService.prototype.addClickListener = function (listener) {
        if (!this._overlayElement) {
            this._overlayElement = document.getElementById('overlay');
        }
        this._overlayElement.addEventListener('click', listener);
    };
    OverlayService.prototype.removeClickListener = function (listener) {
        if (!this._overlayElement) {
            return;
        }
        this._overlayElement.removeEventListener('click', listener);
    };
    OverlayService.prototype.resetElements = function () {
        // Update overlay element
        this._overlayElement.style.visibility = '';
        this._overlayElement.style.transition = '';
        // Update focused element
        this._currentFocusedElement.style.zIndex = '';
        this._currentFocusedElement = null;
        // Update body
        document.body.style.overflow = '';
    };
    OverlayService = __decorate([
        inversify_1.injectable()
    ], OverlayService);
    return OverlayService;
}());
exports.default = OverlayService;
//# sourceMappingURL=overlayService.js.map