"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
var inversify_1 = require("inversify");
var searchResultsComponent_1 = require("../pageHeader/searchResultsComponent");
var SearchWorker = require("worker-loader?inline=true&fallback=false!../workers/search.worker");
var SearchService = /** @class */ (function () {
    function SearchService(searchResultsComponent) {
        this._searchResultsComponent = searchResultsComponent;
    }
    SearchService.prototype.setupSearch = function () {
        var searchWorker = new SearchWorker();
        this.setupWebWorkerSearch(searchWorker);
    };
    ;
    SearchService.prototype.setupWebWorkerSearch = function (searchWorker) {
        var _this = this;
        console.log("using Web Worker");
        // Get index.json url
        var linkElement = document.querySelector('head > link[href*="/index."]');
        var indexRelPath = linkElement.getAttribute('href');
        // Get search data
        var searchDataRequest = new XMLHttpRequest();
        searchDataRequest.open('GET', indexRelPath);
        searchDataRequest.onload = function () {
            // If no index.json is found, just leave the index empty
            if (searchDataRequest.status === 200) {
                searchWorker.postMessage({
                    eventType: 'search-data-received',
                    payload: searchDataRequest.responseText
                });
                // TODO allow query string to perform search on page load
                var searchInputElement = document.getElementById('search-query');
                searchInputElement.
                    addEventListener('keypress', function (event) {
                    // By default, browsers attempt to submit form if enter is pressed
                    if (event.key === 'Enter') {
                        event.preventDefault();
                    }
                });
                searchInputElement.
                    addEventListener('keyup', function (event) {
                    _this._queryString = event.currentTarget.value;
                    if (_this._queryString.length < 1) {
                        _this._searchResultsComponent.setShown(false);
                    }
                    else {
                        // By the time we add this event listener to the search input element,
                        // the search worker is already ready to receive messages.
                        searchWorker.postMessage({ eventType: 'query', payload: _this._queryString });
                    }
                });
            }
        };
        searchDataRequest.send();
        // Generate base URL
        var tempA = document.createElement('a');
        tempA.setAttribute('href', indexRelPath + '/../..'); // Index file is located in <base URL>/resources folder, go two levels up to get to base URL
        var baseUrl = tempA.href;
        baseUrl = baseUrl.substring(0, baseUrl.length - 1); // Drop trailing /
        // Setup listener for query results
        var articleElements = {};
        searchWorker.onmessage = function (event) {
            var items = event.data.payload;
            var result = [];
            for (var i = 0; i < items.length; i++) {
                var item = items[i];
                var articleElement = articleElements[item.relPath];
                if (!articleElement) {
                    articleElement = document.createRange().createContextualFragment(item.snippetHtml).firstChild;
                    // Since index.json is shared by all pages, urls within it are specified as absolute paths (e.g /resources/image.svg).
                    // We need to make these urls absolute for them to work even when the website is published to a path, e.g jering.tech/utilities/<project name>.
                    articleElement.querySelectorAll('*[href^="/"]').forEach(function (element) {
                        element.setAttribute('href', baseUrl + element.getAttribute('href'));
                    });
                    articleElement.querySelectorAll('*[src^="/"]').forEach(function (element) {
                        element.setAttribute('src', baseUrl + element.getAttribute('src'));
                    });
                }
                result.push(articleElement);
            }
            _this._searchResultsComponent.setArticleElements(result, _this._queryString);
        };
    };
    SearchService = __decorate([
        inversify_1.injectable(),
        __metadata("design:paramtypes", [searchResultsComponent_1.default])
    ], SearchService);
    return SearchService;
}());
exports.default = SearchService;
//# sourceMappingURL=searchService.js.map