"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TextInput = /** @class */ (function () {
    function TextInput(_wrapperElement, _inputElement, _clearElement, _onReset) {
        var _this = this;
        this._wrapperElement = _wrapperElement;
        this._inputElement = _inputElement;
        this._clearElement = _clearElement;
        this._onReset = _onReset;
        this.inputKeyUpEventListener = function (event) {
            // Edge returns Esc
            if (event.key === 'Escape' || event.key === 'Esc') {
                _this.reset();
                return;
            }
            _this._clearElement.removeEventListener('transitionend', _this.clearElementTransitionEndListener);
            if (_this._inputElement.value.length > 0) {
                _this.showClearElement();
            }
            else {
                _this.hideClearElement();
            }
        };
        this.clearClickEventListener = function () {
            _this.reset();
            // Keep focus on search input so further searches can be made (no point placing focus on clear button)
            _this._inputElement.focus();
        };
        this.inputFocusOutEventListener = function () {
            // If there is still text, search input is still "active"
            if (_this._inputElement.value.length === 0) {
                _this.deactivateWrapper();
            }
        };
        this.inputFocusEventListener = function () {
            _this.activateWrapper();
        };
        this.activateWrapper = function () {
            _this._wrapperElement.classList.add('active');
        };
        this.deactivateWrapper = function () {
            _this._wrapperElement.classList.remove('active');
        };
        this.clearElementTransitionEndListener = function () {
            _this._clearElement.style.display = 'none';
        };
        _inputElement.addEventListener('focus', this.inputFocusEventListener);
        _inputElement.addEventListener('focusout', this.inputFocusOutEventListener);
        _inputElement.addEventListener('keyup', this.inputKeyUpEventListener);
        _clearElement.addEventListener('click', this.clearClickEventListener);
    }
    TextInput.prototype.reset = function () {
        this._inputElement.value = '';
        this.deactivateWrapper();
        this.hideClearElement();
        if (this._onReset) {
            this._onReset();
        }
    };
    TextInput.prototype.showClearElement = function () {
        this._clearElement.style.display = 'flex';
        this._clearElement.classList.add('active');
    };
    TextInput.prototype.hideClearElement = function () {
        this._clearElement.classList.remove('active');
        this._clearElement.addEventListener('transitionend', this.clearElementTransitionEndListener);
    };
    return TextInput;
}());
exports.default = TextInput;
//# sourceMappingURL=textInput.js.map