"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var lunr = require("lunr");
var lunrBuilder = new lunr.Builder();
// Setup documents
lunrBuilder.ref('relPath');
lunrBuilder.field('title');
lunrBuilder.field('text');
// TODO - https://github.com/olivernn/lunr.js/issues/323
// lunr.tokenizer.separator = /[\s\-\.]+/;
var lunrIndex;
var searchData = {};
onmessage = function (event) {
    switch (event.data.eventType) {
        case 'search-data-received':
            {
                // Build builder
                searchData = JSON.parse(event.data.payload);
                for (var prop in searchData) {
                    lunrBuilder.add(searchData[prop]);
                }
                break;
            }
        case 'query':
            {
                // Generate the index on demand
                if (!lunrIndex) {
                    lunrIndex = lunrBuilder.build();
                }
                var queryString_1 = event.data.payload;
                // Multi-word, partial-word, fuzzy search
                var hits = lunrIndex.query(function (q) {
                    lunr.tokenizer(queryString_1).forEach(function (token) {
                        var subQueryString = token.toString();
                        q.term(subQueryString, { fields: ['title'], usePipeline: true, boost: 200 });
                        // Start and end wild cards, e.g "correct" will match with "rre". Note that wildcard query strings are not passed 
                        // through the search pipeline since they are hard to stem.
                        q.term("*" + subQueryString + "*", { fields: ['title'], usePipeline: true, boost: 100 });
                        // TODO disabled since there is no way to highlight fuzzily matched words
                        // Fuzzy maching, e.g "correct" will match with search term "correcc"
                        // q.term(subQueryString, { fields: ['title'], usePipeline: true, editDistance: 1 })
                        q.term(subQueryString, { fields: ['text'], usePipeline: true, boost: 20 });
                        q.term("*" + subQueryString + "*", { fields: ['text'], usePipeline: true, boost: 10 });
                        // q.term(subQueryString, { fields: ['text'], usePipeline: true, editDistance: 1 })
                    });
                });
                var results_1 = [];
                hits.forEach(function (hit) {
                    var item = searchData[hit.ref];
                    results_1.push(item);
                });
                postMessage({ payload: results_1 });
            }
    }
};
//# sourceMappingURL=search.worker.js.map